@@ -1,4 +1,4 @@
-__version__ = "0.0.1"
+__version__ = "0.0.4"
 
 from multiprocessing import Process, Manager
 import os, sys, time
@@ -8,17 +8,22 @@ Rules = {}
 InitTask = None
 TasksAssignedToProcess = None
 
+ProcessDict = {}
+CommonDict = None
+ProcessInit = None
+
 class Task:
     def __init__(self, TaskType, TaskKey, TaskData):
         self.TaskType = TaskType
         self.TaskKey = TaskKey
         self.TaskData = TaskData
 
-def Host(id, sharedDict):
-    global TasksAssignedToProcess
+def Host(id, sharedDict, commonDict):
+    global TasksAssignedToProcess, CommonDict
 
+    CommonDict = commonDict
     for i in range(1, NGuests + 1):
-        while i not in sharedDict.keys():
+        while i not in sharedDict.keys() or sharedDict[i] not in ("READY", "WAITING"):
             pass
 
     TasksAssignedToProcess = [0] * (NGuests + 1)
@@ -62,7 +67,7 @@ def Host(id, sharedDict):
             for key in list(TaskMap.keys()):
                 if len(TaskMap[key][1]) >= Rules[TaskMap[key][0]][0]:
                     for w, i in sorted(zip(TasksAssignedToProcess[1:], range(1, NGuests + 1))):
-                        if sharedDict[i] == "WAITING":
+                        if sharedDict[i] == "WAITING" and (len(Rules[TaskMap[key][0]]) < 3 or i in Rules[TaskMap[key][0]][2]):
                             sendMessageToGuest(i, Task(TaskMap[key][0], key, TaskMap[key][1])); del TaskMap[key]
                             TasksAssignedToProcess[i] += 1
                             break
@@ -71,7 +76,13 @@ def Host(id, sharedDict):
     for i in range(1, NGuests + 1):
         sendMessageToGuest(i, "EXIT")
 
-def Guest(id, sharedDict):    
+def Guest(id, sharedDict, commonDict): 
+    global CommonDict
+
+    sharedDict[id] = "STARTING"
+    CommonDict = commonDict
+    if ProcessInit is not None:
+        ProcessInit(id)
     sharedDict[id] = "READY"
 
     recvIdx, sendIdx = [0], [0]
@@ -106,7 +117,7 @@ def Guest(id, sharedDict):
             if TASK.TaskType in Rules.keys() or True:
                 Rule = Rules[TASK.TaskType]
                 NewTasks = Rule[1](TASK.TaskKey, TASK.TaskData)
-                if NewTasks is not None and type(NewTasks) == type([]):
+                if NewTasks is not None:
                     for newTask in NewTasks:
                         if type(newTask) == Task:
                             sendMessageToHost(newTask)
@@ -121,14 +132,15 @@ def main():
     assert(type(InitTask.TaskData) == type([]))
 
     sharedDict = Manager().dict()
+    commonDict = Manager().dict()
 
     processList = []
     for i in range(1, NGuests + 1):
-        p = Process(target=Guest, args=(i, sharedDict))
+        p = Process(target=Guest, args=(i, sharedDict, commonDict))
         processList.append(p)
     
     for p in processList:
         p.start()
-    Host(0, sharedDict)
+    Host(0, sharedDict, commonDict)
     for p in processList:
         p.join()
